@@ -68,8 +68,8 @@
 static GConfClient *default_client = NULL;
 static GList *changes = NULL;
 static guint changed_idle;
-#endif
 static GList *listeners = NULL;
+#endif
 
 static gboolean use_system_font = FALSE;
 static PangoFontDescription *titlebar_font = NULL;
@@ -108,17 +108,13 @@ static char *workspace_names[MAX_REASONABLE_WORKSPACES] = { NULL, };
 #ifdef HAVE_GCONF
 static gboolean handle_preference_update_enum (const gchar *key, GConfValue *value);
 
-static gboolean update_window_binding     (const char *name,
-                                           const char *value);
-static gboolean update_screen_binding     (const char *name,
-                                           const char *value);
+static gboolean update_key_binding     (const char *name,
+                                        const char *value);
 static gboolean find_and_update_list_binding (MetaKeyPref *bindings,
                                               const char  *name,
                                               GSList      *value);
-static gboolean update_window_list_binding (const char *name,
-                                            GSList      *value);
-static gboolean update_screen_list_binding (const char *name,
-                                            GSList      *value);
+static gboolean update_key_list_binding (const char *name,
+                                         GSList      *value);
 static gboolean update_command            (const char  *name,
                                            const char  *value);
 static gboolean update_workspace_name     (const char  *name,
@@ -650,8 +646,11 @@ handle_preference_update_enum (const gchar *key, GConfValue *value)
        * we might consider reverting invalid keys to their original values.
        * (We know the old value, so we can look up a suitable string in
        * the symtab.)
+       *
+       * (Empty comment follows so the translators don't see this.)
        */
-      
+
+      /*  */      
       meta_warning (_("GConf key '%s' is set to an invalid value\n"),
                     key);
       return TRUE;
@@ -977,6 +976,7 @@ meta_prefs_remove_listener (MetaPrefsChangedFunc func,
 /* Initialisation.                                                          */
 /****************************************************************************/
 
+#ifdef HAVE_GCONF
 /* @@@ again, use glib's ability to tell you the size of the array */
 static gchar *gconf_dirs_we_are_interested_in[] = {
   "/apps/metacity",
@@ -986,6 +986,7 @@ static gchar *gconf_dirs_we_are_interested_in[] = {
   "/desktop/gnome/interface",
   NULL,
 };
+#endif
 
 void
 meta_prefs_init (void)
@@ -1088,17 +1089,13 @@ change_notify (GConfClient    *client,
   while (preference_update_handler[i]!=NULL)
     {
       if (preference_update_handler[i] (key, value))
-        goto out; /* Get rid of this when we're done with the if */
+        goto out; /* Get rid of this eventually */
 
       i++;
     }
-
-  /* Otherwise, use the enormous if statement. We'll move entries
-   * out of here as it becomes possible to deal with them in a
-   * more general way.
-   */
-
-  if (g_str_has_prefix (key, KEY_WINDOW_BINDINGS_PREFIX))
+  
+  if (g_str_has_prefix (key, KEY_WINDOW_BINDINGS_PREFIX) ||
+      g_str_has_prefix (key, KEY_SCREEN_BINDINGS_PREFIX))
     {
       if (g_str_has_suffix (key, KEY_LIST_BINDINGS_SUFFIX))
         {
@@ -1113,8 +1110,8 @@ change_notify (GConfClient    *client,
 
           list = value ? gconf_value_get_list (value) : NULL;
 
-          if (update_window_list_binding (key, list))
-             queue_changed (META_PREF_WINDOW_KEYBINDINGS);
+          if (update_key_list_binding (key, list))
+            queue_changed (META_PREF_KEYBINDINGS);
         }
       else
         {
@@ -1129,43 +1126,8 @@ change_notify (GConfClient    *client,
 
           str = value ? gconf_value_get_string (value) : NULL;
 
-          if (update_window_binding (key, str))
-             queue_changed (META_PREF_WINDOW_KEYBINDINGS);
-        }
-    }
-  else if (g_str_has_prefix (key, KEY_SCREEN_BINDINGS_PREFIX))
-    {
-      if (g_str_has_suffix (key, KEY_LIST_BINDINGS_SUFFIX))
-        {
-          GSList *list;
-
-          if (value && value->type != GCONF_VALUE_LIST)
-            {
-              meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
-                            key);
-              goto out;
-            }
-
-          list = value ? gconf_value_get_list (value) : NULL;
-
-          if (update_screen_list_binding (key, list))
-             queue_changed (META_PREF_SCREEN_KEYBINDINGS);
-         }
-      else
-        {
-          const char *str;
-
-          if (value && value->type != GCONF_VALUE_STRING)
-            {
-               meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
-                            key);
-               goto out;
-            }
-
-          str = value ? gconf_value_get_string (value) : NULL;
-
-          if (update_screen_binding (key, str))
-             queue_changed (META_PREF_SCREEN_KEYBINDINGS);
+          if (update_key_binding (key, str))
+            queue_changed (META_PREF_KEYBINDINGS);
         }
     }
   else if (g_str_has_prefix (key, KEY_COMMAND_PREFIX))
@@ -1324,15 +1286,16 @@ titlebar_handler (MetaPreference pref,
                   const gchar    *string_value,
                   gboolean       *inform_listeners)
 {
-  PangoFontDescription *new_desc;
+  PangoFontDescription *new_desc = NULL;
 
-  new_desc = pango_font_description_from_string (string_value);
+  if (string_value)
+    new_desc = pango_font_description_from_string (string_value);
 
   if (new_desc == NULL)
     {
       meta_warning (_("Could not parse font description "
                       "\"%s\" from GConf key %s\n"),
-                    string_value,
+                    string_value ? string_value : "(null)",
                     KEY_TITLEBAR_FONT);
 
       *inform_listeners = FALSE;
@@ -1381,7 +1344,7 @@ mouse_button_mods_handler (MetaPreference pref,
   meta_topic (META_DEBUG_KEYBINDINGS,
               "Mouse button modifier has new gconf value \"%s\"\n",
               string_value);
-  if (meta_ui_parse_modifier (string_value, &mods))
+  if (string_value && meta_ui_parse_modifier (string_value, &mods))
     {
       mouse_button_mods = mods;
     }
@@ -1476,16 +1439,17 @@ button_layout_handler (MetaPreference pref,
                          gboolean *inform_listeners)
 {
   MetaButtonLayout new_layout;
-  char **sides;
+  char **sides = NULL;
   int i;
   
   /* We need to ignore unknown button functions, for
    * compat with future versions
    */
   
-  sides = g_strsplit (string_value, ":", 2);
+  if (string_value)
+    sides = g_strsplit (string_value, ":", 2);
 
-  if (sides[0] != NULL)
+  if (sides != NULL && sides[0] != NULL)
     {
       char **buttons;
       int b;
@@ -1545,7 +1509,7 @@ button_layout_handler (MetaPreference pref,
       g_strfreev (buttons);
     }
 
-  if (sides[0] != NULL && sides[1] != NULL)
+  if (sides != NULL && sides[0] != NULL && sides[1] != NULL)
     {
       char **buttons;
       int b;
@@ -1717,11 +1681,8 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_APPLICATION_BASED:
       return "APPLICATION_BASED";
 
-    case META_PREF_SCREEN_KEYBINDINGS:
-      return "SCREEN_KEYBINDINGS";
-
-    case META_PREF_WINDOW_KEYBINDINGS:
-      return "WINDOW_KEYBINDINGS";
+    case META_PREF_KEYBINDINGS:
+      return "KEYBINDINGS";
 
     case META_PREF_DISABLE_WORKAROUNDS:
       return "DISABLE_WORKAROUNDS";
@@ -1816,19 +1777,21 @@ meta_prefs_set_num_workspaces (int n_workspaces)
 }
 
 #define keybind(name, handler, param, flags, stroke, description) \
-  { #name, NULL, flags & BINDING_REVERSES },
-static MetaKeyPref screen_bindings[] = {
-#include "screen-bindings.h"
-  { NULL, NULL, FALSE}
-};
-
-static MetaKeyPref window_bindings[] = {
-#include "window-bindings.h"
+  { #name, NULL, !!(flags & BINDING_REVERSES), !!(flags & BINDING_PER_WINDOW) },
+static MetaKeyPref key_bindings[] = {
+#include "all-keybindings.h"
   { NULL, NULL, FALSE }
 };
 #undef keybind
 
 #ifndef HAVE_GCONF
+
+/**
+ * A type to map names of keybindings (such as "switch_windows")
+ * to the binding strings themselves (such as "<Alt>Tab").
+ * It exists only when GConf is turned off in ./configure and
+ * functions as a sort of ersatz GConf.
+ */
 typedef struct
 {
   const char *name;
@@ -1844,16 +1807,11 @@ typedef struct
  */
 
 #define keybind(name, handler, param, flags, stroke, description) \
-  { #name, keystroke },
+  { #name, stroke },
 
-static MetaSimpleKeyMapping screen_string_bindings[] = {
-#include "screen-bindings.h"
-  { NULL,                                   NULL                         }
-};
-
-static MetaSimpleKeyMapping window_string_bindings[] = {
-#include "window-bindings.h"
-  { NULL,                                   NULL                         }
+static MetaSimpleKeyMapping key_string_bindings[] = {
+#include "all-keybindings.h"
+  { NULL, NULL }
 };
 #undef keybind
 
@@ -1862,81 +1820,44 @@ static MetaSimpleKeyMapping window_string_bindings[] = {
 static void
 init_bindings (void)
 {
-#ifdef HAVE_GCONF
-  int i;
+#ifdef HAVE_GCONF  
+  int i = 0;
   GError *err;
-  
-  i = 0;
-  while (window_bindings[i].name)
+
+  while (key_bindings[i].name)
     {
       GSList *list_val, *tmp;
       char *str_val;
       char *key;
-
-      key = g_strconcat (KEY_WINDOW_BINDINGS_PREFIX, "/",
-                         window_bindings[i].name, NULL);
-
+ 
+      key = g_strconcat (key_bindings[i].per_window?
+                         KEY_WINDOW_BINDINGS_PREFIX:
+                         KEY_SCREEN_BINDINGS_PREFIX,
+                         "/",
+                         key_bindings[i].name, NULL);
+ 
       err = NULL;
       str_val = gconf_client_get_string (default_client, key, &err);
       cleanup_error (&err);
 
-      update_binding (&window_bindings[i], str_val);
+      update_binding (&key_bindings[i], str_val);
 
       g_free (str_val);      
       g_free (key);
 
-      key = g_strconcat (KEY_WINDOW_BINDINGS_PREFIX, "/",
-                         window_bindings[i].name,
+      key = g_strconcat (key_bindings[i].per_window?
+                         KEY_WINDOW_BINDINGS_PREFIX:
+                         KEY_SCREEN_BINDINGS_PREFIX,
+                         "/",
+                         key_bindings[i].name,
                          KEY_LIST_BINDINGS_SUFFIX, NULL);
 
       err = NULL;
 
       list_val = gconf_client_get_list (default_client, key, GCONF_VALUE_STRING, &err);
       cleanup_error (&err);
-
-      update_list_binding (&window_bindings[i], list_val, META_LIST_OF_STRINGS);
-
-      tmp = list_val;
-      while (tmp)
-        {
-          g_free (tmp->data);
-          tmp = tmp->next;
-        }
-      g_slist_free (list_val);
-      g_free (key);
-
-      ++i;
-    }
-
-  i = 0;
-  while (screen_bindings[i].name)
-    {
-      GSList *list_val, *tmp;
-      char *str_val;
-      char *key;
-
-      key = g_strconcat (KEY_SCREEN_BINDINGS_PREFIX, "/",
-                         screen_bindings[i].name, NULL);
-
-      err = NULL;
-      str_val = gconf_client_get_string (default_client, key, &err);
-      cleanup_error (&err);
-
-      update_binding (&screen_bindings[i], str_val);
-
-      g_free (str_val);      
-      g_free (key);
-
-      key = g_strconcat (KEY_SCREEN_BINDINGS_PREFIX, "/",
-                         screen_bindings[i].name,
-                         KEY_LIST_BINDINGS_SUFFIX, NULL);
-
-      err = NULL;
-
-      list_val = gconf_client_get_list (default_client, key, GCONF_VALUE_STRING, &err);
-      cleanup_error (&err);
-
-      update_list_binding (&screen_bindings[i], list_val, META_LIST_OF_STRINGS);
+ 
+      update_list_binding (&key_bindings[i], list_val, META_LIST_OF_STRINGS);
 
       tmp = list_val;
       while (tmp)
@@ -1952,42 +1873,20 @@ init_bindings (void)
 #else /* HAVE_GCONF */
   int i = 0;
   int which = 0;
-  while (window_string_bindings[i].name)
+  while (key_string_bindings[i].name)
     {
-      if (window_string_bindings[i].keybinding == NULL)
-        continue;
-
-      /* Find which window_bindings entry this window_string_bindings entry
-       * corresponds to.
-       */
-      while (strcmp(window_bindings[which].name, 
-                    window_string_bindings[i].name) != 0)
-        which++;
-
-      /* Set the binding */
-      update_binding (&window_bindings[which],
-                      window_string_bindings[i].keybinding);
-
-      ++i;
-    }
-
-  i = 0;
-  which = 0;
-  while (screen_string_bindings[i].name)
-    {
-      if (screen_string_bindings[i].keybinding == NULL)
+      if (key_string_bindings[i].keybinding == NULL) {
+        ++i;
         continue;
+      }
     
-      /* Find which window_bindings entry this window_string_bindings entry
-       * corresponds to.
-       */
-      while (strcmp(screen_bindings[which].name, 
-                    screen_string_bindings[i].name) != 0)
+      while (strcmp(key_bindings[which].name, 
+                    key_string_bindings[i].name) != 0)
         which++;
 
       /* Set the binding */
-      update_binding (&screen_bindings[which], 
-                      screen_string_bindings[i].keybinding);
+      update_binding (&key_bindings[which], 
+                      key_string_bindings[i].keybinding);
 
       ++i;
     }
@@ -2074,7 +1973,7 @@ update_binding (MetaKeyPref *binding,
   MetaVirtualModifier mods;
   MetaKeyCombo *combo;
   gboolean changed;
-  
+
   meta_topic (META_DEBUG_KEYBINDINGS,
               "Binding \"%s\" has new gconf value \"%s\"\n",
               binding->name, value ? value : "none");
@@ -2107,8 +2006,8 @@ update_binding (MetaKeyPref *binding,
 
 #ifdef HAVE_GCONF
    /* Bug 329676: Bindings which can be shifted must not have no modifiers,
-   * nor only SHIFT as a modifier.
-   */
+    * nor only SHIFT as a modifier.
+    */
 
   if (binding->add_shift &&
       0 != keysym &&
@@ -2142,6 +2041,11 @@ update_binding (MetaKeyPref *binding,
                     binding->name,
                     old_setting);
 
+      /* FIXME: add_shift is currently screen_bindings only, but
+       * there's no really good reason it should always be.
+       * So we shouldn't blindly add KEY_SCREEN_BINDINGS_PREFIX
+       * onto here.
+       */
       key = g_strconcat (KEY_SCREEN_BINDINGS_PREFIX, "/",
                          binding->name, NULL);
       
@@ -2345,17 +2249,10 @@ find_and_update_binding (MetaKeyPref *bindings,
 }
 
 static gboolean
-update_window_binding (const char *name,
+update_key_binding (const char *name,
                        const char *value)
 {
-  return find_and_update_binding (window_bindings, name, value);
-}
-
-static gboolean
-update_screen_binding (const char *name,
-                       const char *value)
-{
-  return find_and_update_binding (screen_bindings, name, value);
+  return find_and_update_binding (key_bindings, name, value);
 }
 
 static gboolean
@@ -2388,17 +2285,10 @@ find_and_update_list_binding (MetaKeyPref *bindings,
 }
 
 static gboolean
-update_window_list_binding (const char *name,
+update_key_list_binding (const char *name,
                             GSList *value)
 {
-  return find_and_update_list_binding (window_bindings, name, value);
-}
-
-static gboolean
-update_screen_list_binding (const char *name,
-                            GSList *value)
-{
-  return find_and_update_list_binding (screen_bindings, name, value);
+  return find_and_update_list_binding (key_bindings, name, value);
 }
 
 static gboolean
@@ -2701,20 +2591,12 @@ meta_prefs_get_visual_bell_type (void)
 }
 
 void
-meta_prefs_get_screen_bindings (const MetaKeyPref **bindings,
+meta_prefs_get_key_bindings (const MetaKeyPref **bindings,
                                 int                *n_bindings)
 {
   
-  *bindings = screen_bindings;
-  *n_bindings = (int) G_N_ELEMENTS (screen_bindings) - 1;
-}
-
-void
-meta_prefs_get_window_bindings (const MetaKeyPref **bindings,
-                                int                *n_bindings)
-{
-  *bindings = window_bindings;
-  *n_bindings = (int) G_N_ELEMENTS (window_bindings) - 1;
+  *bindings = key_bindings;
+  *n_bindings = (int) G_N_ELEMENTS (key_bindings) - 1;
 }
 
 MetaActionTitlebar
@@ -2770,10 +2652,10 @@ meta_prefs_get_keybinding_action (const char *name)
 {
   int i;
 
-  i = G_N_ELEMENTS (screen_bindings) - 2; /* -2 for dummy entry at end */
+  i = G_N_ELEMENTS (key_bindings) - 2; /* -2 for dummy entry at end */
   while (i >= 0)
     {
-      if (strcmp (screen_bindings[i].name, name) == 0)
+      if (strcmp (key_bindings[i].name, name) == 0)
         return (MetaKeyBindingAction) i;
       
       --i;
@@ -2793,12 +2675,13 @@ meta_prefs_get_window_binding (const char          *name,
 {
   int i;
 
-  i = G_N_ELEMENTS (window_bindings) - 2; /* -2 for dummy entry at end */
+  i = G_N_ELEMENTS (key_bindings) - 2; /* -2 for dummy entry at end */
   while (i >= 0)
     {
-      if (strcmp (window_bindings[i].name, name) == 0)
+      if (key_bindings[i].per_window &&
+          strcmp (key_bindings[i].name, name) == 0)
         {
-          GSList *s = window_bindings[i].bindings;
+          GSList *s = key_bindings[i].bindings;
 
           while (s)
             {
