@@ -70,7 +70,7 @@ meta_window_release_saved_state (const MetaWindowSessionInfo *info)
 #include <stdio.h>
 #include "main.h"
 #include "util.h"
-#include "display.h"
+#include "display-private.h"
 #include "workspace.h"
 
 static void ice_io_error_handler (IceConn connection);
@@ -807,17 +807,28 @@ save_state (void)
   char *metacity_dir;
   char *session_dir;
   FILE *outfile;
-  GSList *displays;
-  GSList *display_iter;
+  GSList *windows;
+  GSList *tmp;
+  int stack_position;
   
   g_assert (client_id);
 
   outfile = NULL;
   
-  metacity_dir = g_strconcat (g_get_home_dir (), "/.metacity",
+  /*
+   * g_get_user_config_dir() is guaranteed to return an existing directory.
+   * Eventually, if SM stays with the WM, I'd like to make this
+   * something like <config>/window_placement in a standard format.
+   * Future optimisers should note also that by the time we get here
+   * we probably already have full_save_path figured out and therefore
+   * can just use the directory name from that.
+   */
+  metacity_dir = g_strconcat (g_get_user_config_dir (),
+                              G_DIR_SEPARATOR_S "metacity",
                               NULL);
   
-  session_dir = g_strconcat (metacity_dir, "/sessions",
+  session_dir = g_strconcat (metacity_dir,
+                             G_DIR_SEPARATOR_S "sessions",
                              NULL);
 
   if (mkdir (metacity_dir, 0700) < 0 &&
@@ -863,127 +874,116 @@ save_state (void)
   
   fprintf (outfile, "<metacity_session id=\"%s\">\n",
            client_id);
-  
-  displays = meta_displays_list ();
-  display_iter = displays;
-  while (display_iter != NULL)
+
+  windows = meta_display_list_windows (meta_get_display ());
+  stack_position = 0;
+
+  windows = g_slist_sort (windows, meta_display_stack_cmp);
+  tmp = windows;
+  stack_position = 0;
+
+  while (tmp != NULL)
     {
-      GSList *windows;
-      GSList *tmp;
-      int stack_position;
-      
-      windows = meta_display_list_windows (display_iter->data);
-      windows = g_slist_sort (windows, meta_display_stack_cmp);
+      MetaWindow *window;
 
-      stack_position = 0;
-      tmp = windows;
-      while (tmp != NULL)
+      window = tmp->data;
+
+      if (window->sm_client_id)
         {
-          MetaWindow *window;
+          char *sm_client_id;
+          char *res_class;
+          char *res_name;
+          char *role;
+          char *title;
 
-          window = tmp->data;
-
-          if (window->sm_client_id)
-            {
-              char *sm_client_id;
-              char *res_class;
-              char *res_name;
-              char *role;
-              char *title;
-
-              /* client id, class, name, role are not expected to be
-               * in UTF-8 (I think they are in XPCS which is Latin-1?
-               * in practice they are always ascii though.)
-               */
+          /* client id, class, name, role are not expected to be
+           * in UTF-8 (I think they are in XPCS which is Latin-1?
+           * in practice they are always ascii though.)
+           */
               
-              sm_client_id = encode_text_as_utf8_markup (window->sm_client_id);
-              res_class = window->res_class ?
-                encode_text_as_utf8_markup (window->res_class) : NULL;
-              res_name = window->res_name ?
-                encode_text_as_utf8_markup (window->res_name) : NULL;
-              role = window->role ?
-                encode_text_as_utf8_markup (window->role) : NULL;
-              if (window->title)
-                title = g_markup_escape_text (window->title, -1);
-              else
-                title = NULL;
-              
-              meta_topic (META_DEBUG_SM, "Saving session managed window %s, client ID '%s'\n",
-                          window->desc, window->sm_client_id);
-
-              fprintf (outfile,
-                       "  <window id=\"%s\" class=\"%s\" name=\"%s\" title=\"%s\" role=\"%s\" type=\"%s\" stacking=\"%d\">\n",
-                       sm_client_id,
-                       res_class ? res_class : "",
-                       res_name ? res_name : "",
-                       title ? title : "",
-                       role ? role : "",
-                       window_type_to_string (window->type),
-                       stack_position);
-
-              g_free (sm_client_id);
-              g_free (res_class);
-              g_free (res_name);
-              g_free (role);
-              g_free (title);
-              
-              /* Sticky */
-              if (window->on_all_workspaces)
-                fputs ("    <sticky/>\n", outfile);
-
-              /* Minimized */
-              if (window->minimized)
-                fputs ("    <minimized/>\n", outfile);
-
-              /* Maximized */
-              if (META_WINDOW_MAXIMIZED (window))
-		{
-                  fprintf (outfile,
-                           "    <maximized saved_x=\"%d\" saved_y=\"%d\" saved_width=\"%d\" saved_height=\"%d\"/>\n", 
-                           window->saved_rect.x,
-                           window->saved_rect.y,
-                           window->saved_rect.width,
-                           window->saved_rect.height);
-		}
-              
-              /* Workspaces we're on */
-              {
-                int n;
-                n = meta_workspace_index (window->workspace);
-                fprintf (outfile,
-                         "    <workspace index=\"%d\"/>\n", n);
-              }
-
-              /* Gravity */
-              {
-                int x, y, w, h;
-                meta_window_get_geometry (window, &x, &y, &w, &h);
-
-                fprintf (outfile,
-                         "    <geometry x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" gravity=\"%s\"/>\n",
-                         x, y, w, h,
-                         meta_gravity_to_string (window->size_hints.win_gravity));
-              }
-              
-              fputs ("  </window>\n", outfile);
-            }
+          sm_client_id = encode_text_as_utf8_markup (window->sm_client_id);
+          res_class = window->res_class ?
+            encode_text_as_utf8_markup (window->res_class) : NULL;
+          res_name = window->res_name ?
+            encode_text_as_utf8_markup (window->res_name) : NULL;
+          role = window->role ?
+            encode_text_as_utf8_markup (window->role) : NULL;
+          if (window->title)
+            title = g_markup_escape_text (window->title, -1);
           else
-            {
-              meta_topic (META_DEBUG_SM, "Not saving window '%s', not session managed\n",
-                          window->desc);
-            }
-          
-          tmp = tmp->next;
-          ++stack_position;
-        }
-      
-      g_slist_free (windows);
+            title = NULL;
+              
+          meta_topic (META_DEBUG_SM, "Saving session managed window %s, client ID '%s'\n",
+                      window->desc, window->sm_client_id);
 
-      display_iter = display_iter->next;
+          fprintf (outfile,
+                   "  <window id=\"%s\" class=\"%s\" name=\"%s\" title=\"%s\" role=\"%s\" type=\"%s\" stacking=\"%d\">\n",
+                   sm_client_id,
+                   res_class ? res_class : "",
+                   res_name ? res_name : "",
+                   title ? title : "",
+                   role ? role : "",
+                   window_type_to_string (window->type),
+                   stack_position);
+
+          g_free (sm_client_id);
+          g_free (res_class);
+          g_free (res_name);
+          g_free (role);
+          g_free (title);
+              
+          /* Sticky */
+          if (window->on_all_workspaces)
+            fputs ("    <sticky/>\n", outfile);
+
+          /* Minimized */
+          if (window->minimized)
+            fputs ("    <minimized/>\n", outfile);
+
+          /* Maximized */
+          if (META_WINDOW_MAXIMIZED (window))
+            {
+              fprintf (outfile,
+                       "    <maximized saved_x=\"%d\" saved_y=\"%d\" saved_width=\"%d\" saved_height=\"%d\"/>\n", 
+                       window->saved_rect.x,
+                       window->saved_rect.y,
+                       window->saved_rect.width,
+                       window->saved_rect.height);
+            }
+              
+          /* Workspaces we're on */
+          {
+            int n;
+            n = meta_workspace_index (window->workspace);
+            fprintf (outfile,
+                     "    <workspace index=\"%d\"/>\n", n);
+          }
+
+          /* Gravity */
+          {
+            int x, y, w, h;
+            meta_window_get_geometry (window, &x, &y, &w, &h);
+            
+            fprintf (outfile,
+                     "    <geometry x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" gravity=\"%s\"/>\n",
+                     x, y, w, h,
+                     meta_gravity_to_string (window->size_hints.win_gravity));
+          }
+              
+          fputs ("  </window>\n", outfile);
+        }
+      else
+        {
+          meta_topic (META_DEBUG_SM, "Not saving window '%s', not session managed\n",
+                      window->desc);
+        }
+          
+      tmp = tmp->next;
+      ++stack_position;
     }
-  /* don't need to free displays */
-  displays = NULL;
-  
+      
+  g_slist_free (windows);
+
   fputs ("</metacity_session>\n", outfile);
   
  out:
@@ -1061,8 +1061,9 @@ load_state (const char *previous_save_file)
   gsize length;
   char *session_file;
 
-  session_file = g_strconcat (g_get_home_dir (),
-                              "/.metacity/sessions/",
+  session_file = g_strconcat (g_get_user_config_dir (),
+                              G_DIR_SEPARATOR_S "metacity"
+                              G_DIR_SEPARATOR_S "sessions" G_DIR_SEPARATOR_S,
                               previous_save_file,
                               NULL);
 
@@ -1072,11 +1073,32 @@ load_state (const char *previous_save_file)
                             &length,
                             &error))
     {
-      meta_warning (_("Failed to read saved session file %s: %s\n"),
-                    session_file, error->message);
-      g_error_free (error);
-      g_free (session_file);
-      return NULL;
+      char *canonical_session_file = session_file;
+
+      /* Maybe they were doing it the old way, with ~/.metacity */
+      session_file = g_strconcat (g_get_home_dir (),
+                                  G_DIR_SEPARATOR_S ".metacity"
+                                  G_DIR_SEPARATOR_S "sessions"
+                                  G_DIR_SEPARATOR_S,
+                                  previous_save_file,
+                                  NULL);
+      
+      if (!g_file_get_contents (session_file,
+                                &text,
+                                &length,
+                                NULL))
+        {
+          /* oh, just give up */
+
+          meta_warning (_("Failed to read saved session file %s: %s\n"),
+                    canonical_session_file, error->message);
+          g_error_free (error);
+          g_free (session_file);
+          g_free (canonical_session_file);
+          return NULL;
+        }
+
+      g_free (canonical_session_file);
     }
 
   meta_topic (META_DEBUG_SM, "Parsing saved session file %s\n", session_file);
@@ -1169,8 +1191,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <metacity_session> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "metacity_session");
               return;
             }
           
@@ -1244,8 +1266,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <window> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "window");
               session_info_free (pd->info);
               pd->info = NULL;
               return;
@@ -1276,8 +1298,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <window> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "window");
               session_info_free (pd->info);
               pd->info = NULL;
               return;
@@ -1348,8 +1370,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <maximized> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "maximized");
               return;
             }
 
@@ -1408,8 +1430,8 @@ start_element_handler  (GMarkupParseContext *context,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                           _("Unknown attribute %s on <geometry> element"),
-                           name);
+                           _("Unknown attribute %s on <%s> element"),
+                           name, "geometry");
               return;
             }
           
@@ -1678,8 +1700,9 @@ regenerate_save_file (void)
   g_free (full_save_path);
 
   if (client_id)
-    full_save_path = g_strconcat (g_get_home_dir (),
-                                  "/.metacity/sessions/",
+    full_save_path = g_strconcat (g_get_user_config_dir (),
+                                  G_DIR_SEPARATOR_S "metacity"
+                                  G_DIR_SEPARATOR_S "sessions" G_DIR_SEPARATOR_S,
                                   client_id,
                                   ".ms",
                                   NULL);
@@ -1760,9 +1783,8 @@ io_from_warning_dialog (GIOChannel   *channel,
 static void
 warn_about_lame_clients_and_finish_interact (gboolean shutdown)
 {
-  GSList *displays;
-  GSList *display_iter;  
   GSList *lame;
+  GSList *windows;
   char **argv;
   int i;
   GSList *tmp;
@@ -1776,34 +1798,26 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   char timestampbuf[32];
   
   lame = NULL;
-  displays = meta_displays_list ();
-  display_iter = displays;
-  while (display_iter != NULL)
+  windows = meta_display_list_windows (meta_get_display ());
+  tmp = windows;
+  while (tmp != NULL)
     {
-      GSList *windows;
-
-      windows = meta_display_list_windows (display_iter->data);
-      tmp = windows;
-      while (tmp != NULL)
-        {
-          MetaWindow *window;
-
-          window = tmp->data;
-
-          /* only complain about normal windows, the others
-           * are kind of dumb to worry about
-           */
-          if (window->sm_client_id == NULL &&
-              window->type == META_WINDOW_NORMAL)
-            lame = g_slist_prepend (lame, window);
+      MetaWindow *window;
           
-          tmp = tmp->next;
-        }
-      
-      g_slist_free (windows);
+      window = tmp->data;
 
-      display_iter = display_iter->next;
+      /* only complain about normal windows, the others
+       * are kind of dumb to worry about
+       */
+      if (window->sm_client_id == NULL &&
+          window->type == META_WINDOW_NORMAL)
+        lame = g_slist_prepend (lame, window);
+          
+      tmp = tmp->next;
     }
+      
+  g_slist_free (windows);
+  
   if (lame == NULL)
     {
       /* No lame apps. */
@@ -1813,12 +1827,9 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   
   lame = g_slist_sort (lame, (GCompareFunc) windows_cmp_by_title);
 
-  timestamp = meta_display_get_current_time_roundtrip (displays->data);  
+  timestamp = meta_display_get_current_time_roundtrip (meta_get_display ());
   sprintf (timestampbuf, "%u", timestamp);
 
-  /* don't need to free displays */
-  displays = NULL;
-
   len = g_slist_length (lame);
   len *= 2; /* titles and also classes */
   len += 2; /* --timestamp flag and actual timestamp */
